@@ -6,6 +6,8 @@ use crate::history::Command;
 use crate::history_cleaner;
 use crate::settings::KeyScheme;
 use crate::settings::Settings;
+use chrono::{Duration, TimeZone, Utc};
+use humantime::format_duration;
 use std::io::{stdin, stdout, Write};
 use termion::color;
 use termion::event::Key;
@@ -242,6 +244,47 @@ impl<'a> Interface<'a> {
             )
             .unwrap();
 
+            if command.last_run.is_some() {
+                write!(screen, "{}", cursor::Goto(width - 9, index as u16 + RESULTS_TOP_INDEX)).unwrap();
+
+                let duration = &format_duration(
+                    Duration::minutes(
+                        Utc::now()
+                            .signed_duration_since(Utc.timestamp(command.last_run.unwrap(), 0))
+                            .num_minutes(),
+                    )
+                    .to_std()
+                    .unwrap(),
+                )
+                .to_string()
+                .split(" ")
+                .take(2)
+                .map(|s| {
+                    s.replace("years", "y")
+                        .replace("year", "y")
+                        .replace("months", "mo")
+                        .replace("month", "mo")
+                        .replace("days", "d")
+                        .replace("day", "d")
+                        .replace("hours", "h")
+                        .replace("hour", "h")
+                        .replace("minutes", "m")
+                        .replace("minute", "m")
+                        .replace("0s", "< 1m")
+                })
+                .collect::<Vec<String>>()
+                .join(" ");
+
+                let highlight = if self.settings.lightmode {
+                    color::Fg(color::Blue).to_string()
+                } else {
+                    color::Fg(color::LightBlue).to_string()
+                };
+
+                write!(screen, "{}", highlight).unwrap();
+                write!(screen, "{:>9}", duration).unwrap();
+            }
+
             write!(screen, "{}", color::Bg(color::Reset)).unwrap();
             write!(screen, "{}", color::Fg(color::Reset)).unwrap();
         }
@@ -375,7 +418,6 @@ impl<'a> Interface<'a> {
                 return true;
             }
             Key::Ctrl('c')
-            | Key::Ctrl('d')
             | Key::Ctrl('g')
             | Key::Ctrl('z')
             | Key::Esc
@@ -417,7 +459,7 @@ impl<'a> Interface<'a> {
                 self.input.delete(Move::Backward);
                 self.refresh_matches();
             }
-            Key::Delete => {
+            Key::Delete | Key::Ctrl('d') => {
                 self.input.delete(Move::Forward);
                 self.refresh_matches();
             }
@@ -552,9 +594,9 @@ impl<'a> Interface<'a> {
         let mut prev: usize = 0;
         let debug_space = if debug { 90 } else { 0 };
         let max_grapheme_length = if width > debug_space {
-            width - debug_space
+            width - debug_space - 9
         } else {
-            2
+            11
         };
         let mut out = FixedLengthGraphemeString::empty(max_grapheme_length);
 
